# How to install:
- Wer mit SSH Agent unterwegs ist:
```bash
git clone git@git.timmehosting.de:devops/scripte-sammlung/rp-maint.git /root/rpmaint
/bin/bash /root/rpmaint/rp-maint.sh {args}
```
- Per HTTPs
```bash
git clone https://git.timmehosting.de/devops/scripte-sammlung/rp-maint.git /root/rpmaint
```

## Execute
```bash
/bin/bash /root/rpmaint/rp-maint.sh {args}
# oder
/bin/bash /root/rpmaint/upg-logs.sh --order XXXXXX --ticket XXX-XXX-XXXX --mail XXX --nrsv XXXXXXXX
# oder (über ISPCopy Pfad)
/bin/bash /opt/th-ispcopy/lib/rp-maint/upg-logs.sh --order XXXXXX --ticket XXX-XXX-XXXX --mail XXX
```

# rp-maint.sh
Mit dem Script lassen sich alle Webseiten auf einem Server, am ISPConfig vorbei, in den Wartungsmodus oder auch den ReverseProxy für jede Webseite aktivieren.
**Gedachter Einsatz:** Bspw. beim Upgrade eines OpenVZ Servers auf KVM um sich dann, gerade bei vielen Seiten, schnell alle Webseiten in Wartung zu setzen bzw. schnell Proxy für alle Seiten zu aktivieren.

## Usage
- **Wartungsmodus**
  - `./rp-maint.sh --maint`
- **Reverse Proxy**
  - `./rp-maint.sh --proxy ip.ip.ip.ip`
- **Routing** (aktuell nur in Kombination mit ISPCopy)
  - `./rp-maint.sh --routing`
- **Crons löschen**
  - `./rp-maint.sh --clearcrons`

## How does it work ?
Das Script zieht sich direkt aus der ISPConfig Datenbank eines Servers die vorhandenen Web-Domains und deren Details und
- schreibt die vHosts neu
- führt ein `nginx -t` aus
- Startet den nginx einmal durch.  

Die ursprünglichen Konfigurationen schiebt das Scrip dabei nach `/etc/nginx/TIMESTAMP-sites-semi-available/` und via `/bin/mv /etc/nginx/TIMESTAMP-sites-semi-available/* /etc/nginx/sites-available/` kann man die Änderungen auch schnell wieder rückgängig machen, wenn etwas failen sollte.  

## Routing
Das Mail Routing funktioniert im Moment nur in Kombination mit ISPCopy.  
Das Script zieht sich aus der Datei `/root/ispcopy/ispcopy_input.ini` das Ziel für das Mail Routing.  
Das Routing wird dabei komplett unabhängig von ISPConfig erstellt und wird mittels [Nginx Stream Modul](http://nginx.org/en/docs/stream/ngx_stream_core_module.html) umgesetzt.
Im Ablauf wird dabei erst die Konfiguration geschrieben und getestet, dann werden `postfix` und `dovecot` gestoppt, damit keine Interaktion mehr stattfinden kann, dann wird noch ein letzter `rsync` angeschoben und dann wird der nginx durchgestartet und leitet die Anfragen an die jeweiligen [Mail Ports](https://git.timmehosting.de/devops/scripte-sammlung/rp-maint/-/blob/master/rp-maint.sh#L135) durch.

Um das Routing wieder zu deaktivieren // ursprünglichen Zustand zu restoren:
- die Datei `/etc/nginx/nginx_mail_stream.conf` löschen
- `service nginx restart`
- `service postfix start`
- `service dovecot start`

Das Routing setzt die alten IPs ebenfalls die alten IPs auf dem neuen Server als Whitelist inkl. einem Cronjob der die Datei dann nach 30 Tagen löscht.

## Crons aufräumen
Dafür schaut das Script ganz simpel, ob es in `/etc/cron.d/` `ispcxxxxx` und ob es in `/var/spool/crons/crontabs/` `webx` Dateien gibt. Falls ja, dann wird ein `/root/cron_backup_(TIMESTAMP)` Ordner erstellt und die Cron-Dateien werden einfach nach verschoben.

## IP Mapping
Damit IP Mapping funktioniert, muss die Datei `/root/ispcopy/ispcopy_input.ini` vorhanden sein (diese wird eh von der neueren Version von ISPCopy erstell.).  
In der Datei muss bspw. folgendes enthalten sein:
```
target_server_ip=NEUE.HAUPT.IP.ADRESSE
replace_ip_ddd_ddd_ddd_ddd=aaa.aaa.aaa.aaa
replace_ip_eee_eee_eee_eee=bbb.bbb.bbb.bbb
```
Dann kann das Script auch das IP-Mapping
- **Reverse Proxy IP Mapping**
  - `./rp-maint.sh --ispcopy_ini`

## Was zu beachten?
Es könnte eventuell bei MirrorServern bzw. bei alten Clouds knartschen. Da müsste man vielleicht nochmal nachschauen. Wobei MirrorServer aber meist nur als NS2 vom Kunden genutzt werden sollten, ohne dass sich darauf Webseiten befinden.  

# upg-logs.sh
`/bin/bash /root/rpmaint/upg-logs.sh --order XXXXXX --ticket XXX-XXX-XXXX --mail XXX --nrsv XXXXXXXX` ...
wertet die auf dem Server liegenden access.log's aus und verschickt das Resultat als Mail an den Kunden und an unser HESK.
Das Script kann auf jedem beliebigen Server ausgeführt werden.
Der Content der Mail ist aber stark auf die OVZ-Upgrades ausgerichtet und müsste in anderen Fällen auch angepasst werden.  

**Pflichtparams**
Ohne `--order XXXXXX --ticket XXX-XXX-XXXX --mail XXX` kann das Script in der Form nicht richtig arbeiten und spuckt dann auch entsprechende Fehlermeldungen aus.
Falls `--nrsv XXXXXXXX` fehlt, dann schaut das Script, ob es eine von ISPCopy angelegte `input_ini`-Datei findet und zieht sich den Server von da.
Falls beides nicht vorhanden ist, dann steigt das Script ebenfalls aus.
 
Auf einem Server der also frisch von ISPCopy verarbeitet wurde, reicht ein `--order XXXXXX --ticket XXX-XXX-XXXX --mail XXX` aus.

```bash
/bin/bash /root/rpmaint/upg-logs.sh --order 123456 --ticket TE5-GM9-J84J --mail it+dien@timmehosting.de
16.11.2023 15:22:04 [UPG Log Checks] Passwordfile fetched with correct login.
16.11.2023 15:22:04 [UPG Log Checks] Installing Cronjob /etc/cron.d/ovz_upg_sni-2-test_logchecks
16.11.2023 15:22:04 [UPG Log Checks] Successfully installed Cronjob.
16.11.2023 15:22:05 [UPG Log Checks] Sending mail to: it+dien@timmehosting.de
16.11.2023 15:22:09 [UPG Log Checks] Success - Statuscode: 0
16.11.2023 15:22:09 [UPG Log Checks] Sending mail to: support@timmehosting.de
16.11.2023 15:22:09 [UPG Log Checks] Success - Statuscode: 0
```

Das Script kann auch im Nachgang gerne noch von Hand ausgeführt werden. Dabei wird aber der Cronjob nicht mehr verändert.
Ist nützlich, wenn der Kunde einen "aktuellen Zwischenstand" haben will, dann könnte man es direkt mit `--order XXXXXX --ticket XXX-XXX-XXXX --mail XXX` ausführen und gerne auch eine andere E-Mail-Adresse angeben (für die Agentur bspw.). Dann wird aber nur einmalig die Mail an die abweichende Mail-Adresse verschickt.

## Cronjob
Bei erstem Ausführen wird auch ein Cronjob installiert, der das Script dann genau so auch regelmäßig startet.  
Der Cron läuft Montag, Mittwoch und Freitag um 03:00 Uhr und wertet die Logs des Vortags aus. Der Schedule wurde so gewählt weil:
- der Kunde hat die Mail dann schon zum Frühstück im Postfach
- der Vortag, weil man da einen vollen Tag Traffic hat
- um 3:00 Uhr, weil das CP alte Server um 01:00 Uhr abschaltet bzw. um 02:00 Uhr löscht und so läuft der Cron garantiert nicht nochmal unmittelbar vorm Abschalten des Servers.

## Mails?
Das Script verschickt E-Mails an die E-Mail-Adresse die mit `--mail` übergeben wurde und an unsere `support@` Adresse.
Mails gehen als `nrpl@timmehosting.de` per `SMTP` mit `AUTH` über unseren `mx` raus.
Dazu liegt auf `ovzm.hostianer.com` eine `/root/upg_traffic_info` die das SMTP Passwort hält und diese Datei wird dann bei Installation einmal per SSH via Agent gezogen und dann lokal auf dem Server unter `/root/upg_logs_passfile` gespeichert.
 
Die Mails haben den Betreff `Re: [#${ticket}] $(hostname -f)-Upgrade` und weil das HESK ein korrektes `Re: [#${ticket}]` schon ausreicht, wird die Mail an uns auch direkt in das Ticket sortiert.  
 
Die Mails an den Kunden und an uns sind offensichtlich unterschiedlich. Wir bräuchten den ganzen Teil mit Anrede, und Footer nicht und weil das HESK ein `reply-to` nicht richtig versteht, müssen so oder so zwei Mails verschickt werden.
 
Die Mail an uns verursacht zwar, dass das Ticket dann dreimal in der Woche wieder aufplopt, kann aber auch ein nützlicher Hinweis auf "Ja, da läuft ja noch was" sein.
Ein Beispiel der Mail an den Kunden findet man im `it@` Postfach mit dem Betreff `TE5-GM9-J84J` und die Variante an uns im HESK => `TE5-GM9-J84J`
