#!/bin/bash
function call() {
  echo -e "$(date '+%d.%m.%Y %H:%M:%S') [UPG Log Checks] $@"
}

if [[ ! $(which idn) ]]; then
  call "please install idn pakage"
  call "apt install idn"
  call "and then call me again"
  exit 0
fi

mail_id=$(pwgen -s 25 1)
new_server=''
order=''
old_server=$(hostname -f)
send=0
ispc_ini="/root/ispcopy/ispcopy_input.ini"
logchecks_cron_file="/etc/cron.d/ovz_upg_$(hostname)_logchecks"
app_dir="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)"
app_name="$(basename "${BASH_SOURCE[0]}")"
app_path="${app_dir}/${app_name}"
m_user='nrpl@timmehosting.de'
m_pass_file="/root/upg_logs_passfile"
ssh_opt='-o StrictHostKeyChecking=no -o LogLevel=QUIET -o BatchMode=yes'
t_user=$(logname)
r_user='root'
logdate=$(LC_TIME=en_US.UTF-8 date "+%d/%b/%Y" -d "yesterday")
echodate=$(LC_TIME=de_DE.UTF-8 date "+%d. %b %Y" -d "yesterday")
old_ip=$(ifconfig | grep -v '127.0\|172.16' | awk '/inet /{print$2}' | xargs)

srv_id=$(cat /usr/local/ispconfig/server/lib/config.inc.php | grep 'server_id' | grep -Eo '[[:digit:]]+')
if [[ -z ${srv_id} ]]; then
  call "Not able to tell the server_id. Exiting here!"
  exit 1
fi

if [[ ! -f ${m_pass_file} ]]; then
  if [[ $(echo "${t_user}" | grep -o 'timme') != 'timme' ]]; then
    call "You need to run this with your timme user!"
    exit 0
  fi
  mpw=$(ssh ${ssh_opt} ${t_user}@ovzm.hostianer.com "sudo cat /root/upg_traffic_info")
  if [[ ! -z ${mpw} ]]; then
    if [[ $(curl --silent --ssl-reqd --url 'smtps://mx.timmehosting.de' --user "${l_user}:${mpw}") != "67" ]]; then
      echo -e "${mpw}" > "${m_pass_file}"
      chmod 600 ${m_pass_file}
      if [[ -f ${m_pass_file} ]] && [[ $(cat ${m_pass_file}) == $(echo -e "${mpw}") ]]; then
        send='1'
        m_pass=$(cat ${m_pass_file})
        call "Passwordfile fetched with correct login."
        ## We just installed the Script. So we want todays traffic.
        logdate=$(LC_TIME=en_US.UTF-8 date "+%d/%b/%Y")
        echoxdate=$(LC_TIME=de_DE.UTF-8 date "+%d. %b %Y")
        echodate=$(LC_TIME=de_DE.UTF-8 date "+%d. %b %Y")
        
      fi
    fi
  fi
else
  m_pass=$(cat ${m_pass_file})
  send='1'
fi

if [[ $# = 0 ]]; then
  call "Please provide at least: --order xxxxxx --ticket xxx-xxx-xxxx --mail customer@mail.tld"
  exit 1
fi
while [[ $# -ge 1 ]]; do
  INPUT="$1"
  case $INPUT in
    --order)
      order="${2}"
      if [[ -z ${order} ]]; then
        call "Please provide --order XXXXX"
        exit 1
      elif [[ $(echo ${order} | grep -Eo '[0-9]{6}') != "${order}" ]]; then
        call "Please provide --order XXXXX"
        exit 1
      fi
      shift
      shift
      ;;
    --ticket)
      ticket="${2}"
      if [[ -z ${ticket} ]]; then
        call "Please provide --tid xxx-xxx-xxxx"
        exit 1
      elif [[ $(echo ${ticket} | grep -Eo '^[[:alnum:]]{3}-[[:alnum:]]{3}-[[:alnum:]]{4}$') != "${ticket}" ]]; then
        call "Please provide --ticket xxx-xxx-xxxx"
        exit 1
      fi
      shift
      shift
      ;;
    --nsrv)
      new_server=${2}
      shift
      shift
      ;;
    --mail)
      cmail=${2}
      if [[ -z ${cmail} ]]; then
        call "Please provide --mail customer@mail.tld"
        exit 1
      elif [[ $(echo ${cmail} | grep -Eo '^[A-Za-z0-9._%+-]+@[A-Za-z0-9.-]+\.[A-Za-z]{2,15}$') != "${cmail}" ]]; then
        call "Please provide --mail customer@mail.tld"
        exit 1
      fi
      shift
      shift
      ;;
    *)
    call "Wrong input: $INPUT"
    exit 1
    ;;
  esac
done


if [[ -z ${order} || -z ${ticket} || -z ${cmail}  ]]; then
  call "Please provide at least: --order xxxxxx --ticket xxx-xxx-xxxx --mail customer@mail.tld"
  exit 1
fi

if [[ ! -f ${ispc_ini} && -z ${new_server} ]]; then
  call "Please provide --nsrv new.server.tld"
  exit 1
fi
if [[ -z ${new_server} && -f ${ispc_ini} ]]; then
  new_server=$(awk -F '=' '/target_server_hostname/{print$2}' ${ispc_ini})
  if [[ -z ${new_server} ]]; then
    call "Something went wrong parsing file: ${ispc_ini}"
    exit 1
  fi
fi

if [[ -f ${ispc_ini} ]]; then
  new_ip=$(awk -F '=' '/target_server_ip/{print$2}' ${ispc_ini})
else
  new_ip=$(dig A ${new_server} @8.8.8.8 +short)
  if [[ $(echo -e "${new_ip}" | wc -l) -ge 2 ]]; then
    ## we dont exptect more than one IP per hostname
    new_ip=''
  fi
fi

if [[ -z ${new_ip} ]]; then
  call "Cant tell the new IP."
  call "Please check: 'dig A ${new_server} @8.8.8.8 +short'"
  call "Continuing either way ... "
else
  new_ip="(${new_ip})"
fi

if [[ -z ${old_ip} ]]; then
  call "Cant tell the old IP."
  call "Please check with 'ifconfig'"
  call "Continuing either way ... "
else
  old_ip="(${old_ip})"
fi


if [[ ! -f ${logchecks_cron_file} ]]; then
  if [[ $(echo "${t_user}" | grep -o 'timme') != 'timme' ]]; then
    call "You need to run this with your timme user!"
    exit 0
  fi
  call "Installing Cronjob ${logchecks_cron_file}"
  logchecks_cron_string="PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin
MAILTO=''
SHELL='$(which bash)'

0 03 * * 1,3,5 root $(which bash) ${app_path} --order ${order} --ticket ${ticket} --mail ${cmail} --nsrv ${new_server} > /dev/null"
  echo -e "${logchecks_cron_string}" > ${logchecks_cron_file}
  if [[ -f ${logchecks_cron_file} ]]; then
    if [[ $(cat ${logchecks_cron_file} | grep "${ticket}" | wc -l ) -gt 0 ]]; then
      call "Successfully installed Cronjob."
      send='1'
    fi
  fi
fi


mail_head="From: Support - TimmeHosting.de <${m_user}>
To: ${cmail}
Reply-To: Support - TimmeHosting.de <support@timmehosting.de>
Subject: Re: [#${ticket}] ${old_server}-Upgrade"

mail_body="
##### ############################## #####
- Traffic-Info ${old_server}
- Upgrade ${old_server} noch nicht erledigt
##### ############################## #####
####REPLACE####
----------------->%-----------------
- Auftrag: ${order}
- Ticket: ${ticket}
- Alter Server: ${old_server} ${old_ip}
- Neuer Server: ${new_server} ${new_ip}
-----------------%<-----------------

Auswertung erstellt am: $(LC_TIME=de_DE.UTF-8 date "+%d. %b %Y")
Log-Daten sind vom: ${echodate}
----------------->%-----------------
### Domains mit access.log
$(for i in $(find /var/www/ -maxdepth 1 -type l | sed 's/\/var\/www\///g' | grep '^[A-Za-z0-9]'); do d=$(echo -e "${i}" | idn -u); th=$(dig NS ${i} @8.8.8.8 | grep 'SOA\|IN' | grep 'timmehosting' | wc -l); [[ ${th} -gt '0' ]] && ns="TH" || ns="EXT"; echo -e "D: ${i}; NS: ${ns}; IPs: $(dig A ${i} +short @8.8.8.8 | awk '$0 ~ /^[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+$/ { print $0}' | xargs); HITS: $(cat /var/log/ispconfig/httpd/${i}/access.log | grep -c "${logdate}")"; done | column -t -s';')

### Domains ohne access.log (Alias/SubDomains) (Traffic kann leider nicht ausgewertet werden)
$(for i in $(mysql --defaults-extra-file=/etc/mysql/debian.cnf dbispconfig -e "select domain,vhost_type from web_domain WHERE server_id = ${srv_id};" | awk '/NULL/{print$1}'); do d=$(echo -e "${i}" | idn -u); th=$(dig NS ${i} @8.8.8.8 | grep 'SOA\|IN' | grep 'timmehosting' | wc -l); [[ ${th} -gt '0' ]] && ns="TH" || ns="EXT"; echo -e "D: ${d}; NS: ${ns}; IPs: $(dig A ${i} +short @8.8.8.8 | awk '$0 ~ /^[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+$/ { print $0}' | xargs)"; done | column -t -s';')
-----------------%<-----------------
"
mail_footer="
[NS: TH]  Nameserver bei TimmeHosting.
[NS: EXT] Nameserver nicht bei TimmeHosting.
[IPs]     IP Adressen, die im DNS eines Public Resolver zum Zeitpunkt der Auswertung ausgeliefert wurden.
[HITS]    Anzahl der Zeilen im access.log der jeweiligen Seite, vom ${echodate}, zum Zeitpunkt der Messung.

Für den Fall, dass das Upgrade abgeschlossen werden soll, antworten Sie gerne direkt auf diese E-Mail indem Sie die Antwortfunktion ihres E-Mail-Programms verwenden.
Diese E-Mail wurde automatisch auf dem Server ${old_server} generiert.

---
Timme Hosting GmbH & Co. KG
Ovelgönner Weg 43
21335 Lüneburg
Deutschland

Tel.: +49 (0) 4131 22 78 1-0
Fax: +49 (0) 4131 22 78 1-78
Email: support@timmehosting.de
URL: https://timmehosting.de

Amtsgericht Lüneburg HRA 201911
Ust-ID-Nr.: DE291340442

Persönlich haftende Gesellschafterin:
Timme Hosting Verwaltungs GmbH
Amtsgericht Lüneburg HRB 204577
Geschäftsführer: Falko Timme

Weitere Projekte:
ISPConfig: http://www.ispconfig.org
HowtoForge: http://www.howtoforge.com

---------------------------------------

Diese E-Mail enthält vertrauliche und/oder rechtlich geschützte
Informationen. Wenn Sie nicht der richtige Adressat sind oder diese
E-Mail irrtümlich erhalten haben, informieren Sie bitte sofort den
Absender und vernichten Sie diese Mail. Das unerlaubte Kopieren sowie
die unbefugte Weitergabe dieser Mail und/oder der Inhalte dieser Mail
ist nicht gestattet.

This e-mail may contain confidential and/or privileged information. If
you are not the intended recipient (or have received this e-mail in
error) please notify the sender immediately and destroy this e-mail.Any
unauthorised copying, disclosure or distribution of the material of this
e-mail is strictly forbidden.
ID: ${mail_id}"

mail_why='\nFolgendes Upgrade ist aktuell noch nicht abgeschlossen. Weiter unten finden Sie eine Auswertung der Log-Dateien auf dem alten Server.\nWenn der alte Server nicht weiter benötigt ist, lassen Sie es uns gerne wissen und wir schließen das Upgrade innerhalb unserer Geschäftszeiten ab.\n'


if [[ ${send} == '1' ]]; then
  mailfile=$(mktemp /root/upg_logs_tempmail_XXXX)
  touch "${mailfile}"
  touch "${mailfile}x"
  
  ## Client Mail
  echo -e "${mail_head} ${mail_body} ${mail_footer}" | sed "s/####REPLACE####/${mail_why}/g"  > "${mailfile}"
  mail_why="\n### Diese Mail ist nur eine BCC-Kopie an uns. ###\n### ID: ${mail_id} ###\n"

  ## Hesk Mail
  echo -e "${mail_head} ${mail_body}" | sed "s/From: Support - TimmeHosting.de/From: ${old_server}/g" | sed "{:q;N;s/####REPLACE####/${mail_why}/g;t q}" | sed '/Reply-To/d' > "${mailfile}x"

  if [[ $(cat ${mailfile} | wc -l) -gt 2 ]] && [[ ${send} == '1' ]]; then
    ## Client Mail
    call "Sending mail to: ${cmail}"
    err=''
    curl --silent --ssl-reqd --url 'smtps://mx.timmehosting.de' --user "${m_user}:${m_pass}" --mail-from "${m_user}" --mail-rcpt ${cmail} --upload-file "${mailfile}"
    err=$?
    if [[ ${err} == 0 ]]; then
      call "Success - Statuscode: ${err}"
      rm -f "${mailfile}"
    else
      call "Something went wrong - Statuscode: ${err}"
    fi
    ## Hesk Mail
    err=''
    call "Sending mail to: support@timmehosting.de"
    curl --silent --ssl-reqd --url 'smtps://mx.timmehosting.de' --user "${m_user}:${m_pass}" --mail-from "${m_user}" --mail-rcpt 'support@timmehosting.de' --upload-file "${mailfile}x"
    err=$?
    if [[ ${err} == 0 ]]; then
      call "Success - Statuscode: ${err}"
      rm -f "${mailfile}x"
    else
      call "Something went wrong - Statuscode: ${err}"
    fi
  fi
fi
