<?php

define('ISPC_RSYNC_PROGRESS', 1);

class ssh_lib extends lib {
	
	private $hostname;
	private $port = 22;
	private $username = 'root';
	private $use_identity_file = true;
	private $identity_file = "/root/.ssh/id_rsa_migration";
	private $local_rsync_version = false;
	private $remote_rsync_version = false;
	
	public function __construct($hostname = '', $port = 22, $username = 'root', $identity_file = null) {
		$this->hostname = $hostname;
		$this->port = $port;
		$this->username = $username;
		if($identity_file && file_exists($identity_file)) {
			$this->identity_file = $identity_file;
		}
        if(!file_exists($this->identity_file)) {
            echo("Create migration SSH key\n");
            exec("ssh-keygen -q -N '' -b 4096 -t rsa -f {$this->identity_file}");
        }
        chmod($this->identity_file, 0600);
        chmod("{$this->identity_file}.pub", 0600);
	}
	
	
	protected function log($message, $priority = 0) {
		// Priority: 0 = debug, 1 = warn, 2 = error.
		if(defined('APP_NAME') && APP_NAME == 'ISPCopy') {
			global $icp;
			$icp->log($message, $priority);
		} elseif(class_exists('ISPConfigLog')) {
			ISPConfigLog::info($message);
		} else {
			// TODO: Implement logging
			if($priority >= $this->getLogLevel()) {
				$this->do_echo($message."\n");
			}
		}
	}
	
	protected function do_echo($msg) {
		echo $msg;
	}
	
	public function setHostname($hostname) {
		$this->hostname = $hostname;
	}
	
	public function setPort($port) {
		$this->port = $port;
	}
	
	public function setUsername($username) {
		$this->username = $username;
	}

	public function getHostname() {
		return $this->hostname;
	}
	
	public function getPort() {
		return $this->port;
	}
	
	public function getUsername() {
		return $this->username;
	}

    public function getIdentityFile() {
        return $this->identity_file;
    }

	private function checkSettings() {
		if($this->hostname == '') {
			$this->log('Hostname not set',2);
			return false;
		}
		return true;
	}


	//* Test the SSH connection
	public function testConnect() {
		
		// Test the settings first
		if(!$this->checkSettings()) return false;
		
		// Check connection without identity file
		exec('ssh -p '.$this->port.' -oStrictHostKeyChecking=no -oConnectTimeout=5 -oBatchMode=yes '.escapeshellarg($this->username.'@'.$this->hostname).' cat /etc/hosts > /dev/null 2>&1',$output,$return_var);
		if($return_var === 0) {
			$this->use_identity_file = true;
			return true;
		} else {
			// Do an SSH test connect with identity file
			exec('ssh -p '.$this->port.' -oStrictHostKeyChecking=no -oConnectTimeout=5 -oBatchMode=yes -i '.$this->identity_file.' '.escapeshellarg($this->username.'@'.$this->hostname).' cat /etc/hosts > /dev/null 2>&1',$output,$return_var);
			if($return_var === 0) {
				$this->use_identity_file = true;
				return true;
			} else {
				return false;
			}
		}
	}
	
	//* Install a cert based SSH login
	public function installSSHCertDEP() {
			if(!file_exists($this->identity_file)) {
				$this->do_echo("Creating SSH key.\n");
				passthru('ssh-keygen -q -N \'\' -b 4096 -t rsa -f '.$this->identity_file);
			}
			$this->do_echo("Now we transfer the key to the target server.\n");
			$check = exec('ssh-copy-id 2>&1 | head -n 1');
			if(strpos($check, '-p port') !== false) {
				passthru('ssh-copy-id -i '.$this->identity_file.' -p '.$this->port.' -oStrictHostKeyChecking=no -oConnectTimeout=5 '. escapeshellarg($this->username.'@'.$this->hostname));
			} else {
				passthru('ssh-copy-id -i '.$this->identity_file.' ' . escapeshellarg('-p '.$this->port.' -oStrictHostKeyChecking=no -oConnectTimeout=5 '.$this->username.'@'.$this->hostname));
			}
			$this->use_identity_file = true;
	}

    public function installSSHCert($username = "") {
        $this->do_echo("Copy root ssh key.");
        $tmp = $this->username;
        if(!empty($username)) {
            $this->username = $username;
        }
        exec("ssh-keygen -f \"/root/.ssh/known_hosts\" -R \"{$this->hostname}\" >> /dev/null 2>&1");
        $pub_key = trim(file_get_contents("{$this->identity_file}.pub"));
        $this->exec("echo '{$pub_key}' | sudo tee -a /root/.ssh/authorized_keys2");
        exec("ssh-keygen -f \"/root/.ssh/known_hosts\" -R \"{$this->hostname}\" >> /dev/null 2>&1");
        $this->username = $tmp;
    }

    public function create_deafaults_extra_file() {
        if(!$this->isFile($this->mysql_defaults_extra_file)) {
            $tmpfile = tempnam(sys_get_temp_dir(), "mysql-");
            $this->getFile('/usr/local/ispconfig/server/lib/mysql_clientdb.conf', $tmpfile);
            include $tmpfile;
            $content = <<<TXT
[client]
host=$clientdb_host
user=$clientdb_user
password='$clientdb_password'
TXT;
            if(!$this->isDir($this->getLogDir())) {
                $this->mkdir($this->getLogDir());
            }
            $this->putFileContents($this->mysql_defaults_extra_file, $content);
            unlink($tmpfile);
        }
    }

	//* get a file from the remote server
	public function getFile($remote, $local) {
		$identity = ($this->use_identity_file)?'-i '.$this->identity_file:'';
		exec('ssh -p '.$this->port.' -oStrictHostKeyChecking=no -oConnectTimeout=5 -oBatchMode=yes '.$identity.' '.escapeshellarg($this->username.'@'.$this->hostname).' "cat '.escapeshellarg($remote).'" > '.escapeshellarg($local),$output,$return_var);
		if($return_var === 0) {
			// check if the file was really transferred
			if(file_exists($local)) {
				$this->log('File '.$local.' successfully transferred.',0);
				return true;
			} else {
				$this->log('Error: File '.$local.' transfer failed.',1);
				return false;
			}
		} else {
			$this->log('SSH Error: File '.$local.' could not be transferred.',1);
			return false;
		}
	}
	
	//* Send a file to the remote server
	public function putFile($local, $remote, $compress = false, $is_dir = false) {
		if(!file_exists($local)) {
			$this->log('Error: File '.$local.' does not exist.',1);
			return false;
		}
		$identity = ($this->use_identity_file)?'-i '.$this->identity_file:'';
		exec('scp -p' . ($compress ? ' -C' : '') . ($is_dir ? ' -r' : '') . ' -P ' . $this->port. ' -o StrictHostKeyChecking=no -o ConnectTimeout=5 -o BatchMode=yes '.$identity.' '.escapeshellarg($local) . ' ' . escapeshellarg($this->username.'@'.$this->hostname . ':' . $remote), $output, $return_var);
		//exec('cat '.escapeshellarg($local).' | ssh -p '.$this->port.' -oStrictHostKeyChecking=no -oConnectTimeout=5 -oBatchMode=yes '.$identity.' '.escapeshellarg($this->username.'@'.$this->hostname).' "cat > '.escapeshellarg($remote).'"',$output,$return_var);
		if($return_var === 0) {
			$this->log('File '.$remote.' successfully transferred.',0);
			return true;
		} else {
			$this->log('Error: File '.$remote.' transfer failed.',1);
			return false;
		}
	}
	
	//* Get a file content from remote server
	public function getFileContents($remote) {
		$local = tempnam(sys_get_temp_dir(), "ssh");
		$identity = ($this->use_identity_file)?'-i '.$this->identity_file:'';
		exec('ssh -p '.$this->port.' -oStrictHostKeyChecking=no -oConnectTimeout=5 -oBatchMode=yes '.$identity.' '.escapeshellarg($this->username.'@'.$this->hostname).' "cat '.escapeshellarg($remote).'" > '.escapeshellarg($local),$output,$return_var);
		if($return_var === 0) {
			// check if the file was really transferred
			if(file_exists($local)) {
				$this->log('File '.$local.' successfully transferred.',0);
				$content = file_get_contents($local);
				unlink($local);
				return $content;
			} else {
				$this->log('Error: File '.$local.' transfer failed.',1);
				return false;
			}
		} else {
			$this->log('SSH Error: File '.$local.' could not be transferred.',1);
			return false;
		}
	}
	
	//* Write file on remote server
	public function putFileContents($remote,$content,$mode = 'replace') {
		$local = tempnam(sys_get_temp_dir(), "ssh");
		file_put_contents($local,$content);
		$identity = ($this->use_identity_file)?'-i '.$this->identity_file:'';
		if($mode == 'replace') {
			// Replace the file
			$copycmd = '>';
		} else {
			// append content to file
			$copycmd = '>>';
		}
		exec('cat '.escapeshellarg($local).' | ssh -p '.$this->port.' -oStrictHostKeyChecking=no -oConnectTimeout=5 -oBatchMode=yes '.$identity.' '.escapeshellarg($this->username.'@'.$this->hostname).' "cat '.$copycmd.' '.escapeshellarg($remote).'"',$output,$return_var);
		unlink($local);
		if($return_var === 0) {
			$this->log('File '.$remote.' successfully transferred.',0);
			return true;
		} else {
			$this->log('Error: File '.$remote.' transfer failed.',1);
			return false;
		}
	}
	
	//* Copy file on remote server
	public function cp($from_path, $to_path, $options = '-prf') {
		$identity = ($this->use_identity_file)?'-i '.$this->identity_file:'';
		exec('ssh -p '.$this->port.' -oStrictHostKeyChecking=no -oConnectTimeout=5 -oBatchMode=yes '.$identity.' '.escapeshellarg($this->username.'@'.$this->hostname).' "cp '.$options.' '.escapeshellarg($from_path).' '.escapeshellarg($to_path).'"',$output,$return_var);
		if($return_var === 0) {
			$this->log('File '.$from_path.' successfully copied.',0);
			return true;
		} else {
			$this->log('Error: File '.$from_path.' copy failed.',1);
			return false;
		}
	}
	
	//* move file on remote server
	public function mv($from_path, $to_path, $options = '-f') {
		$identity = ($this->use_identity_file)?'-i '.$this->identity_file:'';
		exec('ssh -p '.$this->port.' -oStrictHostKeyChecking=no -oConnectTimeout=5 -oBatchMode=yes '.$identity.' '.escapeshellarg($this->username.'@'.$this->hostname).' "mv '.$options.' '.escapeshellarg($from_path).' '.escapeshellarg($to_path).'"',$output,$return_var);
		if($return_var === 0) {
			$this->log('File '.$from_path.' successfully moved.',0);
			return true;
		} else {
			$this->log('Error: File '.$from_path.' move failed.',1);
			return false;
		}
	}
	
	//* Check if a file exists
	public function isFile($path) {
		$identity = ($this->use_identity_file)?'-i '.$this->identity_file:'';
		exec('ssh -p '.$this->port.' -oStrictHostKeyChecking=no -oConnectTimeout=5 -oBatchMode=yes '.$identity.' '.escapeshellarg($this->username.'@'.$this->hostname).' "test -e '.escapeshellarg($path).'"',$output,$return_var);
		if($return_var === 0) {
			$this->log('File '.$path.' exists.',0);
			return true;
		} else {
			$this->log('File '.$path.' does not exist.',0);
			return false;
		}
	}

    public function isLink(string $path) : bool {
        $identity = ($this->use_identity_file)?'-i '.$this->identity_file:'';
        exec('ssh -p '.$this->port.' -oStrictHostKeyChecking=no -oConnectTimeout=5 -oBatchMode=yes '.$identity.' '.escapeshellarg($this->username.'@'.$this->hostname).' "find '.escapeshellarg($path).' -type l -wholename '.escapeshellarg($path).'"',$output,$return_var);
        if(empty($output)) {
            $this->log('File '.$path.' is no symlink.',0);
            return false;
        } else {
            $this->log('File '.$path.' is a symlink.',0);
            return true;
        }
    }
	
	
	public function mkdir($path, $mode = '0750') {
		$identity = ($this->use_identity_file)?'-i '.$this->identity_file:'';
		exec('ssh -p '.$this->port.' -oStrictHostKeyChecking=no -oConnectTimeout=5 -oBatchMode=yes '.$identity.' '.escapeshellarg($this->username.'@'.$this->hostname).' "mkdir -p -m ' . escapeshellarg($mode) . ' '.escapeshellarg($path).'"',$output,$return_var);
		if($return_var === 0) {
			$this->log('Directory '.$path.' created.',0);
			return true;
		} else {
			$this->log('Directory '.$path.' not created.',0);
			return false;
		}
	}
	
	//* Check if a directory exists
	public function isDir($path) {
		$identity = ($this->use_identity_file)?'-i '.$this->identity_file:'';
		exec('ssh -p '.$this->port.' -oStrictHostKeyChecking=no -oConnectTimeout=5 -oBatchMode=yes '.$identity.' '.escapeshellarg($this->username.'@'.$this->hostname).' "test -d '.escapeshellarg($path).'"',$output,$return_var);
		if($return_var === 0) {
			$this->log('Directory '.$path.' exists.',0);
			return true;
		} else {
			$this->log('Directory '.$path.' does not exist.',0);
			return false;
		}
	}
	
	//* Execute command on the remote server
	public function exec($cmd, $returncodes_ok = array(), &$return_code = null) {
		$identity = ($this->use_identity_file)?'-i '.$this->identity_file:'';
		exec('ssh -p '.$this->port.' -oStrictHostKeyChecking=no -oConnectTimeout=5 -oBatchMode=yes '.$identity.' '.escapeshellarg($this->username.'@'.$this->hostname).' '.escapeshellarg($cmd),$output,$return_var);
		if($return_var === 0 || (!empty($returncodes_ok) && is_array($returncodes_ok) && in_array($return_var, $returncodes_ok, true))) {
			$this->log('Successfully executed command '.$cmd,0);
			$return_code = $return_var;
			return implode("\n",$output);
		} else {
			$this->log('Command '.$cmd.' failed with code ' . $return_var . '.',0);
			$return_code = $return_var;
			return false;
		}
	}
	
	//* Test MySQL login credentials on remote server
	public function testDBLogin($hostname,$username,$password) {
		if($password != '') {
			$result = $this->exec('mysql --defaults-extra-file='.$this->mysql_defaults_extra_file.' -e "select now()" 2> /dev/null');
		} else {
			$result = $this->exec('mysql --defaults-extra-file='.$this->mysql_defaults_extra_file.' -e "select now()" 2> /dev/null');
		}
		if($result === false) {
			return false;
		} else {
			return true;
		}
	}
	
	
	//* Copy files by rsync or scp
	public function copyFiles($from_path, $to_path, $mode = 'rsync', $delete = true, $flags = false, string $additional_options = null) {
		// rsync mode
		if($mode == 'rsync') {
			if(!$this->local_rsync_version) {
				exec('rsync --version 2>/dev/null', $output, $return_var);
				if($return_var === 0 && preg_match('/rsync\s+version\s+(\d+(?:\.\d+)*)\s/i', $output[0], $match)) {
					$this->local_rsync_version = $match[1];
				}
			}
			if(!$this->remote_rsync_version) {
				$tmp = $this->exec('rsync --version 2>/dev/null');
				if($tmp !== false && preg_match('/rsync\s+version\s+(\d+(?:\.\d+)*)\s/i', $tmp, $match)) {
					$this->remote_rsync_version = $match[1];
				}
			}
			
			if($flags & ISPC_RSYNC_PROGRESS) {
				// we need to check capability
				if(version_compare($this->local_rsync_version, '3.1.0', '<') || version_compare($this->remote_rsync_version, '3.1.0', '<')) {
					$flags = $flags & ~ISPC_RSYNC_PROGRESS;
					$this->log('Cannot display rsync progress as either local or remote version is lower than 3.1.0.', 0);
				}
			}
			$delete_option = ($delete == true)?'--delete-during':'';
			$identity = ($this->use_identity_file)?'-i '.$this->identity_file:'';
			$add_options = '';
			if($flags & ISPC_RSYNC_PROGRESS) $add_options .= ' --info=progress2';
			$cmd = 'rsync -a';
			if(!($flags & ISPC_RSYNC_PROGRESS)) $cmd .= 'q';
			if($additional_options) {
				$cmd .=  trim(str_replace("-", "", $additional_options));
			}
			
			// do not copy Jailkits, see https://git.timmehosting.de/devops/ispconfig/ispconfig_migration-tool/-/issues/13
			//$excludes = " --exclude 'clients/client*/web*/.bash_history' --exclude 'clients/client*/web*/.bashrc' --exclude 'clients/client*/web*/bin' --exclude 'clients/client*/web*/dev' --exclude 'clients/client*/web*/etc' --exclude 'clients/client*/web*/lib' --exclude 'clients/client*/web*/lib64' --exclude 'clients/client*/web*/opt' --exclude 'clients/client*/web*/.profile' --exclude 'clients/client*/web*/usr' --exclude 'clients/client*/web*/var'"; // alternative Schreibweise, funktioniert
			//$excludes = ' --exclude="clients/client*/web*/.bash_history" --exclude="clients/client*/web*/.bashrc" --exclude="clients/client*/web*/bin" --exclude="clients/client*/web*/dev" --exclude="clients/client*/web*/etc" --exclude="clients/client*/web*/lib" --exclude="clients/client*/web*/lib64" --exclude="clients/client*/web*/opt" --exclude="clients/client*/web*/.profile" --exclude="clients/client*/web*/usr" --exclude="clients/client*/web*/var"'; // alternative Schreibweise, funktioniert
			$excludes = " --exclude='clients/client*/web*/.bash_history' --exclude='clients/client*/web*/.bashrc' --exclude='clients/client*/web*/bin' --exclude='clients/client*/web*/dev' --exclude='clients/client*/web*/etc' --exclude='clients/client*/web*/lib' --exclude='clients/client*/web*/lib64' --exclude='clients/client*/web*/opt' --exclude='clients/client*/web*/.profile' --exclude='clients/client*/web*/usr' --exclude='clients/client*/web*/var' --exclude='apps/phpmyadmin' --exclude='apps/webmail'";
			
			$cmd .= ' '.$delete_option.$add_options.$excludes.' -e "ssh -p '.$this->port.' -oStrictHostKeyChecking=no -oConnectTimeout=10 -oBatchMode=yes '.$identity.'" '.escapeshellarg($from_path).' '.escapeshellarg($this->username.'@'.$this->hostname.':'.$to_path);
			if($flags & ISPC_RSYNC_PROGRESS) $cmd .= ' 2>/dev/null';
			else $cmd .= ' >/dev/null 2>&1';
			if($flags & ISPC_RSYNC_PROGRESS) {
				print "\n";
				passthru($cmd, $return_var);
			} else {
				exec($cmd,$output,$return_var);
			}
			if($return_var === 0 || $return_var === 23 || $return_var === 24) {
				$this->log('Rsync copy from '.$from_path.' to ' . $to_path . ' successful.',0);
				return true;
			} else {
				$this->log('Error: Rsync copy from '.$from_path.' to ' . $to_path . ' failed. Return value '.$return_var.'.',1);
				$this->log('Command was: ' . $cmd, 0);
				return false;
			}
		}
		// TODO: scp mode
		return false;
	}

	public function datalogUpdate($tablename, $update_data, $index_field, $index_value, $create_datalog = true) {
		include '/usr/local/ispconfig/server/lib/mysql_clientdb.conf';

		$old_rec = $this->queryAllRecords("SELECT * FROM $tablename WHERE $index_field = \"$index_value\"");
		if(is_array($update_data)) {
			$link = @mysqli_connect($clientdb_host, $clientdb_user, $clientdb_password, '', $clientdb_port);

			$update_str = '';
			foreach ($update_data as $key => $val) {
				if($key != $index_field) {
					$update_str .= "`" . $key . "`=\"" . $val . "\",";
				}
			}
			$update_str = substr($update_str, 0, -1);
			$link->close();
		} else {
			$update_str = $update_data;
		}
		$this->performInsert("UPDATE `$tablename` SET $update_str WHERE $index_field = \"$index_value\"", false, false);
		if($create_datalog) {
			$new_rec = $this->queryAllRecords("SELECT * FROM $tablename WHERE $index_field = \"$index_value\"");
			$this->datalogSave($tablename, "UPDATE", $index_field, $index_value, $old_rec[0], $new_rec[0]);
		}
	}

	protected function datalogSave($db_table, $action, $primary_field, $primary_id, $record_old, $record_new, $force_update = false) {
		include '/usr/local/ispconfig/server/lib/mysql_clientdb.conf';

		// Insert backticks only for incomplete table names.
		if(stristr($db_table, '.')) {
			$escape = '';
		} else {
			$escape = '`';
		}

		if($force_update == true) {
			//* We force a update even if no record has changed
			$diffrec_full = array('new' => $record_new, 'old' => $record_old);
			$diff_num = count((array)$record_new);
		} else {
			//* get the difference record between old and new record
			$tmp = $this->diffrec($record_old, $record_new);
			$diffrec_full = $tmp['diff_rec'];
			$diff_num = $tmp['diff_num'];
			unset($tmp);
		}

		// Insert the server_id, if the record has a server_id
		$server_id = (isset($record_old['server_id']) && $record_old['server_id'] > 0)?$record_old['server_id']:0;
		if(isset($record_new['server_id'])) $server_id = $record_new['server_id'];


		if($diff_num > 0) {
			$diffstr = serialize($diffrec_full);
			if(isset($_SESSION['s']['user']['username'])) {
				$username = $_SESSION['s']['user']['username'];
			} else {
				$username = 'admin';
			}
			$dbidx = $primary_field.':'.$primary_id;

			if($action == 'INSERT') $action = 'i';
			if($action == 'UPDATE') $action = 'u';
			if($action == 'DELETE') $action = 'd';

			$link = @mysqli_connect($clientdb_host, $clientdb_user, $clientdb_password, '', $clientdb_port);
			$diffstr = mysqli_real_escape_string($link, $diffstr);
			$link->close();

//            $diffstr = str_replace("\"", "\\\"", $diffstr);
			$sql = "INSERT INTO sys_datalog (dbtable,dbidx,server_id,action,tstamp,user,data) VALUES ('%s', '%s', %d, '%s', %d, '%s', '%s')";
			$this->performInsert(sprintf($sql, $db_table, $dbidx, $server_id, $action, time(), $username, $diffstr), false, true);
		}

		return true;
	}

	private function diffrec($record_old, $record_new) {
		$diffrec_full = array();
		$diff_num = 0;

		if(is_array($record_old) && count($record_old) > 0) {
			foreach($record_old as $key => $val) {
				// if(!isset($record_new[$key]) || $record_new[$key] != $val) {
				if(@$record_new[$key] != $val) {
					// Record has changed
					$diffrec_full['old'][$key] = $val;
					$diffrec_full['new'][$key] = @$record_new[$key];
					$diff_num++;
				} else {
					$diffrec_full['old'][$key] = $val;
					$diffrec_full['new'][$key] = $val;
				}
			}
		} elseif(is_array($record_new)) {
			foreach($record_new as $key => $val) {
				if(isset($record_new[$key]) && @$record_old[$key] != $val) {
					// Record has changed
					$diffrec_full['new'][$key] = $val;
					$diffrec_full['old'][$key] = @$record_old[$key];
					$diff_num++;
				} else {
					$diffrec_full['new'][$key] = $val;
					$diffrec_full['old'][$key] = $val;
				}
			}
		}

		return array('diff_num' => $diff_num, 'diff_rec' => $diffrec_full);
	}

	public function queryAllRecords($sql) {
		global $conf;

		$tmp_results = $this->exec('mysql --defaults-extra-file=' . $this->mysql_defaults_extra_file . " " . $conf['db_database'] . ' -e "' . $sql . '";');
		$results = explode("\n", $tmp_results);

		$data = [];
		if(!empty($results)) {
			$headers = [];
			foreach (explode("\t", $results[0]) as $col) {
				$headers[] = $col;
			}
			for ($i = 1; $i < count($results); $i++) {
				$row = [];
				$cols = explode("\t", $results[$i]);
				for ($c = 0; $c < count($cols); $c++) {
					$row[$headers[$c]] = $cols[$c];
				}
				$data[] = $row;
			}
		}
		return $data;
	}

	private function performInsert($sql, $local = true, $insert = true) {
		global $conf, $ssh;

		$tmp_sql_file = "/tmp/tmp_insert.sql";
		file_put_contents($tmp_sql_file, $sql . ($insert?";SELECT LAST_INSERT_ID();":";SELECT ROW_COUNT();"));
		$cmd = "mysql --defaults-extra-file=" . $this->mysql_defaults_extra_file . " " . $conf['db_database'] . " -sN < " . $tmp_sql_file;
		$id = 0;
		if($local) {
			exec($cmd, $output);
			if(!empty($output)) {
				$id = $output[0];
			}
		} else {
			$ssh->putFile($tmp_sql_file, $tmp_sql_file);
			$id = $ssh->exec($cmd);
			$ssh->exec("rm -rf " . $tmp_sql_file);
		}

		preg_match("/^[0-9]+$/", $id, $id_check);
		if(!empty($id_check) && (int)$id > 0) {
			$id = (int)$id;
		} else {
			$id = 0;
			$err_name = str_replace(".sql", "", str_replace("/tmp", "/root/ispcopy/err", $tmp_sql_file));
			$i = 0;
			while(file_exists($err_name . "-" . $i . ".sql")) {
				$i++;
			}
			$err_name = $err_name . "-" . $i . ".sql";
			$err_dir = explode("/", $err_name);
			array_pop($err_dir);
			exec("mkdir -p " . implode("/", $err_dir) . " && cp $tmp_sql_file " . $err_name);
		}
		unlink($tmp_sql_file);
		return $id;
	}
	
	
}

/*

SSH Error codes

	0
 	Success 
   	1 
 	Undetermined error in file copy 
   	3 
 	Destination is not directory, but it should be 
   	4 
 	Connecting to host failed 
   	5 
 	Connection lost for some reason 
   	6 
 	File does not exist 
   	7 
 	No permission to access file 
   	8 
 	Undetermined error from sshfilexfer 
   	9 
 	File transfer protocol mismatch 
   	255 
 	Error occurred in SSH 


RSYNC Exit values

EXIT VALUES 
 ������0 �����Success 

 ������1 �����Syntax or usage error 

 ������2 �����Protocol incompatibility 

 ������3 �����Errors selecting input/output files, dirs 

 ������4 �����Requested �action not supported: an attempt was made to manipulate 64-bit files on a platform that can- 
 �������������not support them; or an option was specified that is supported by the client and not by the server. 

 ������5 �����Error starting client-server protocol 

 ������6 �����Daemon unable to append to log-file 

 ������10 ����Error in socket I/O 

 ������11 ����Error in file I/O 

 ������12 ����Error in rsync protocol data stream 

 ������13 ����Errors with program diagnostics 

 ������14 ����Error in IPC code 

 ������20 ����Received SIGUSR1 or SIGINT 

 ������21 ����Some error returned by waitpid() 

 ������22 ����Error allocating core memory buffers 

 ������23 ����Partial transfer due to error 

 ������24 ����Partial transfer due to vanished source files 

 ������25 ����The --max-delete limit stopped deletions 

 ������30 ����Timeout in data send/receive

 ������35 ����Timeout waiting for daemon connection





*/